
#if !defined(COMSTL_INCL_COMSTL_HPP_STRING_ACCESS)
# error This file cannot be directly included, and should only be included within comstl/string_access.hpp
#endif /* !COMSTL_INCL_COMSTL_HPP_STRING_ACCESS */

#if !defined(STLSOFT_UNITTEST)
# error This file cannot be included when not unit-testing STLSoft
#endif /* !STLSOFT_UNITTEST */

namespace unittest
{
    namespace
    {
        ss_bool_t test_comstl_string_access_GUID(unittest_reporter *r)
        {
            ss_bool_t   bSuccess = true;

            /* Check GUID conversion */
            GUID const  guid    =   GUID_NULL;

            if(0 != wcscmp(c_str_ptr(guid), L"{00000000-0000-0000-0000-000000000000}"))
            {
                r->report("c_str_ptr(GUID) failed", __LINE__);
                bSuccess = false;
            }

            if(0 != wcscmp(c_str_ptr_null(guid), L"{00000000-0000-0000-0000-000000000000}"))
            {
                r->report("c_str_ptr_null(GUID) failed", __LINE__);
                bSuccess = false;
            }

            if(COMSTL_CCH_GUID != c_str_len(guid))
            {
                r->report("c_str_len(GUID) failed", __LINE__);
                bSuccess = false;
            }

            return bSuccess;
        }

        ss_bool_t test_comstl_string_access_VARIANT(unittest_reporter *r)
        {
            ss_bool_t   bSuccess = true;

            /* Check VARIANT conversion */
            VARIANT     var1;

            ::VariantInit(&var1);
            var1.vt     =   VT_I4;
            var1.lVal   =   1968;

            if(0 != wcscmp(L"1968", c_str_ptr(var1)))
            {
                r->report("c_str_ptr(VARIANT) failed", __LINE__);
                bSuccess = false;
            }

            if(0 != wcscmp(L"1968", c_str_ptr_null(var1)))
            {
                r->report("c_str_ptr_null(VARIANT) failed", __LINE__);
                bSuccess = false;
            }

            VARIANT     var2;

            ::VariantInit(&var1);

            if(0 != wcscmp(L"", c_str_ptr(var2)))
            {
                r->report("c_str_ptr(VARIANT) failed", __LINE__);
                bSuccess = false;
            }

            if(NULL != c_str_ptr_null(var2))
            {
                r->report("c_str_ptr_null(VARIANT) failed", __LINE__);
                bSuccess = false;
            }

            return bSuccess;
        }

        ss_bool_t test_comstl_string_access(unittest_reporter *r)
        {
            using stlsoft::unittest::unittest_initialiser;

            ss_bool_t               bSuccess    =   true;

            unittest_initialiser    init(r, "COMSTL", "string_access", __FILE__);

            // 1. Test GUID
            if(bSuccess)
            {
                bSuccess = test_comstl_string_access_GUID(r);
            }

            // 1. Test CWindow
            if(bSuccess)
            {
                bSuccess = test_comstl_string_access_VARIANT(r);
            }

            return bSuccess;
        }


        unittest_registrar    unittest_comstl_string_access(test_comstl_string_access);
    } // anonymous namespace

} // namespace unittest

