/* /////////////////////////////////////////////////////////////////////////////
 * File:        winstl/file_functions.hpp
 *
 * Purpose:     Helper functions for file handling
 *
 * Created:     1st January 2005
 * Updated:     22nd September 2005
 *
 * Home:        http://stlsoft.org/
 *
 * Copyright (c) 2005, Matthew Wilson and Synesis Software
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * - Redistributions of source code must retain the above copyright notice, this
 *   list of conditions and the following disclaimer.
 * - Redistributions in binary form must reproduce the above copyright notice,
 *   this list of conditions and the following disclaimer in the documentation
 *   and/or other materials provided with the distribution.
 * - Neither the name(s) of Matthew Wilson and Synesis Software nor the names of
 *   any contributors may be used to endorse or promote products derived from
 *   this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 * ////////////////////////////////////////////////////////////////////////// */


/// \file winstl/file_functions.hpp
///
/// Helper functions for file handling.

#ifndef WINSTL_INCL_WINSTL_HPP_FILE_FUNCTIONS
#define WINSTL_INCL_WINSTL_HPP_FILE_FUNCTIONS

#ifndef STLSOFT_DOCUMENTATION_SKIP_SECTION
# define WINSTL_VER_WINSTL_HPP_FILE_FUNCTIONS_MAJOR     1
# define WINSTL_VER_WINSTL_HPP_FILE_FUNCTIONS_MINOR     1
# define WINSTL_VER_WINSTL_HPP_FILE_FUNCTIONS_REVISION  3
# define WINSTL_VER_WINSTL_HPP_FILE_FUNCTIONS_EDIT      9
#endif /* !STLSOFT_DOCUMENTATION_SKIP_SECTION */

/* ////////////////////////////////////////////////////////////////////////////
 * Includes
 */

#ifndef WINSTL_INCL_H_WINSTL
# include "winstl.h"
#endif /* !WINSTL_INCL_H_WINSTL */

#ifndef WINSTL_INCL_WINSTL_HPP_FILESYSTEM_TRAITS
# include <winstl/filesystem_traits.hpp>
#endif /* !WINSTL_INCL_WINSTL_HPP_FILESYSTEM_TRAITS */
#ifndef WINSTL_INCL_WINSTL_HPP_PROCESSHEAP_ALLOCATOR
# include <winstl/processheap_allocator.hpp>
#endif /* !WINSTL_INCL_WINSTL_HPP_PROCESSHEAP_ALLOCATOR */
#ifndef STLSOFT_INCL_STLSOFT_HPP_AUTO_BUFFER
# include <stlsoft/auto_buffer.hpp>
#endif /* !STLSOFT_INCL_STLSOFT_HPP_AUTO_BUFFER */
#ifndef STLSOFT_INCL_STLSOFT_HPP_STRING_ACCESS
# include <stlsoft/string_access.hpp>
#endif /* !STLSOFT_INCL_STLSOFT_HPP_STRING_ACCESS */
#ifndef STLSOFT_INCL_STLSOFT_HPP_SCOPED_HANDLE
# include <stlsoft/scoped_handle.hpp>
#endif /* !STLSOFT_INCL_STLSOFT_HPP_SCOPED_HANDLE */
#ifndef STLSOFT_INCL_STLSOFT_HPP_STRING_TRAITS
# include <stlsoft/string_traits.hpp>
#endif /* !STLSOFT_INCL_STLSOFT_HPP_STRING_TRAITS */

/* /////////////////////////////////////////////////////////////////////////////
 * Namespace
 */

#ifndef _WINSTL_NO_NAMESPACE
# if defined(_STLSOFT_NO_NAMESPACE) || \
     defined(STLSOFT_DOCUMENTATION_SKIP_SECTION)
/* There is no stlsoft namespace, so must define ::winstl */
namespace winstl
{
# else
/* Define stlsoft::winstl_project */

namespace stlsoft
{

namespace winstl_project
{

# endif /* _STLSOFT_NO_NAMESPACE */
#endif /* !_WINSTL_NO_NAMESPACE */

/* /////////////////////////////////////////////////////////////////////////////
 * Functions
 */

template<   ss_typename_param_k S1
        ,   ss_typename_param_k S2
        >
inline ws_uint64_t load_text_file(S1 const &fileName, S2 &contents)
{
    typedef string_traits<S1>                       traits_t;
    typedef ss_typename_type_k traits_t::value_type C;

    scoped_handle<HANDLE>   h(  filesystem_traits<C>::create_file(  stlsoft_ns_qual(c_str_ptr)(fileName)
                                                                ,   GENERIC_READ
                                                                ,   FILE_SHARE_READ
                                                                ,   NULL
                                                                ,   OPEN_EXISTING
                                                                ,   0
                                                                ,   NULL)
                            ,   &filesystem_traits<C>::close_handle
                            ,   INVALID_HANDLE_VALUE);

    ws_uint64_t             size    =   filesystem_traits<C>::get_file_size(::stlsoft::get_handle(h));

    if( 0 != size &&
        ~0 != size)
    {
        if(size > 0xFFFFFFFF)
        {
            throw std::out_of_range("Cannot read in files larger than 4GB");
        }
        else
        {
            typedef ::stlsoft::auto_buffer<C, processheap_allocator<C>, 1024>   buffer_t;

            buffer_t    buffer(size);
            DWORD       dw;

            if(!::ReadFile(::stlsoft::get_handle(h), &buffer[0], buffer.size(), &dw, NULL))
            {
                return ~ws_uint64_t(0);
            }
            else
            {
                contents.assign(&buffer[0], dw);

                return size;
            }
        }
    }

    return size;
}

/* ////////////////////////////////////////////////////////////////////////// */

#ifndef _WINSTL_NO_NAMESPACE
# if defined(_STLSOFT_NO_NAMESPACE) || \
     defined(STLSOFT_DOCUMENTATION_SKIP_SECTION)
} // namespace winstl
# else
} // namespace winstl_project
} // namespace stlsoft
# endif /* _STLSOFT_NO_NAMESPACE */

#endif /* !_WINSTL_NO_NAMESPACE */

/* ////////////////////////////////////////////////////////////////////////// */

#endif /* !WINSTL_INCL_WINSTL_HPP_FILE_FUNCTIONS */

/* ////////////////////////////////////////////////////////////////////////// */
