
// Created  23rd February 2002
// Updated: 21st May 2005

#include <MWRegUtl.h>
#include <MLCompId.h>

#include <stlsoft.h>

#include <winstl.h>
#include <winstl/system_version.h>
#include <winstl/filesystem_traits.h>

#include <stdio.h>

#include <algorithm>
#include <numeric>
#include <string>
#include <list>

#include <Win32/PendingFileOperations.hpp>

/* Usage
 *
 * no arguments - show the current pending operations
 *
 * -a src dest
 * -d n
 * -e n src dest
 * -m n-from n-to
 */

#include "showver.h"

#include <MLCompId.h>
#include "MBldHdr.h"
////////////////////////////////////////////////////////////////////////////////

#if defined(__STLSOFT_COMPILER_IS_BORLAND)
# pragma warn -8084
#endif /* __STLSOFT_COMPILER_IS_BORLAND */

////////////////////////////////////////////////////////////////////////////////

const int	_mccVerHi	=	__SYV_MAJOR;
const int	_mccVerLo	=	__SYV_MINOR;
const int	_mccVerRev	=	__SYV_REVISION;
const int	_mccBldNum	=	__SYV_BUILDNUMBER;

#define	COMPANY_NAME	"Synesis Software"
#define	TOOL_NAME		"Pending File Operation Editing"
#define	EXE_NAME		"pfoped"

////////////////////////////////////////////////////////////////////////////////

using ::winstl::system_version;

typedef	::winstl::filesystem_traits<char>				filesystem_traits_t;

typedef SynesisWin::PendingFileOperations::string_type	string_t;

typedef SynesisWin::PendingFileOperations::value_type	entry_t;

typedef SynesisWin::PendingFileOperations				container_t;

////////////////////////////////////////////////////////////////////////////////

namespace Operation
{
	enum Operation
	{
			usage
		,	list
		,	addition
		,	deletion
		,	edit
		,	find
		,	movement
	};
} // namespace Operation

////////////////////////////////////////////////////////////////////////////////

void load_entries(container_t &list);
void save_entries(const container_t &list);

static void usage(int bExit, char const *reason);
bool show_list(int highlightIndex = -1);
bool edit_item(size_t from, const char *src, const char *dest);
bool find_item(const char *src);
bool delete_item(size_t from);
bool add_item(const char *src, const char *dest);
bool move_item(int from, int to);

////////////////////////////////////////////////////////////////////////////////

int main(int argc, char *argv[])
{
	int						iRet		=	0;
	int						i;
	const char				*src		=	NULL;
	const char				*dest		=	NULL;
	int						from		=	-1;
	int						to			=	-1;
	Operation::Operation	operation	=	Operation::usage;
	bool					bShowUsage	=	false;
	char					szSrc[_MAX_PATH + 1];
	char					szDest[_MAX_PATH + 1];

	if(!system_version::winnt())
	{
		fprintf(stderr, "%s can only be used on Windows NT family operating systems\n", EXE_NAME);

		exit(EXIT_FAILURE);
	}
	else
	{
		if(argc == 1)
		{
			operation = Operation::list;
		}
		else for(i = 1; i < argc; ++i)
		{
			const char	*arg	=	argv[i];

			if(arg[0] == '-')
			{
				if(arg[1] == '-')
				{
					/* -- arguments */
#ifdef _SYNSOFT_INTERNAL_BUILD
					if(0 == strcmp("--version", arg))
					{
						return show_version();
					}
					else
#endif /* _SYNSOFT_INTERNAL_BUILD */
					{
						usage(1, "Invalid argument");
					}
				}
				else
				{
					/* - arguments */
					switch(arg[1])
					{
						case	'?':
							usage(1, NULL);
							break;
						case	'a':
							operation = Operation::addition;
							break;
						case	'd':
							operation = Operation::deletion;
							break;
						case	'e':
							operation = Operation::edit;
							break;
						case	'f':
							operation = Operation::find;
							break;
						case	'l':
							operation = Operation::list;
							break;
						case	'm':
							operation = Operation::movement;
							break;
						default:
							usage(1, "Invalid argument");
							break;
					}
				}
			}
			else
			{
				/* other arguments */
				switch(operation)
				{
					case	Operation::list:
						i = argc;
						break;
					case	Operation::addition:
						if(src == NULL)
						{
							src = arg;
						}
						else if(dest == NULL)
						{
							dest = arg;
						}
						else
						{
							operation = Operation::usage;
						}
						break;
					case	Operation::deletion:
						if(from == -1)
						{
							from = strtol(arg, NULL, 0);
						}
						else
						{
							operation = Operation::usage;
						}
						break;
					case	Operation::edit:
						if(from == -1)
						{
							from = strtol(arg, NULL, 0);
						}
						else if(src == NULL)
						{
							src = arg;
						}
						else if(dest == NULL)
						{
							dest = arg;
						}
						else
						{
							operation = Operation::usage;
						}
						break;
					case	Operation::find:
						if(src == NULL)
						{
							src = arg;
						}
						else
						{
							operation = Operation::usage;
						}
						break;
					case	Operation::movement:
						if(from == -1)
						{
							from = strtol(arg, NULL, 0);
						}
						else if(to == -1)
						{
							to = strtol(arg, NULL, 0);
						}
						else
						{
							operation = Operation::usage;
						}
						break;
					case	Operation::usage:
					default:
						stlsoft_assert(0);
						break;
				}
			}
		}

		// Now validate the src and dest
		//
		// (i) Make sure that src exists
		//
		// (ii) Make sure that src is a file
		//
		// (iii) Make sure that dest is not an existing directory. If it
		// is, then need to extract the
		//
		// (iv) make sure that they are on the same drive
		//

		if( NULL != dest &&
			*dest == '\0')
		{
			dest = NULL;
		}

		if(NULL == dest)
		{
			switch(operation)
			{
				case	Operation::usage:
				case	Operation::list:
				case	Operation::deletion:
				case	Operation::movement:
					break;
				case	Operation::addition:
				case	Operation::find:
					break;
				case	Operation::edit:
					operation = Operation::usage;
					break;
			}
		}

		if( !bShowUsage &&
			(	operation == Operation::edit ||
				operation == Operation::addition))
		{
			// (i) Source must exist
			if(!filesystem_traits_t::file_exists(src))
			{
				fprintf(stderr, "The source - \"%s\" - does not exist\n\n", src);

				bShowUsage = true;
			}
			else
			{
				// (ii) Source cannot be a directory
				if(	operation != Operation::addition &&
					filesystem_traits_t::is_directory(src))
				{
					fprintf(stderr, "The source - \"%s\" - is a directory. Please specify a file\n\n", src);

					bShowUsage = true;
				}
				else
				{
					// (iii) Append source filename if dest is a directory
					LPTSTR	lpFilePartSrc;

					filesystem_traits_t::get_full_path_name(src, stlsoft_num_elements(szSrc), szSrc, &lpFilePartSrc);
					src = szSrc;
					if(NULL != dest)
					{
						filesystem_traits_t::get_full_path_name(dest, stlsoft_num_elements(szDest), szDest);
						dest = szDest;

						if(filesystem_traits_t::is_directory(szDest))
						{
							filesystem_traits_t::ensure_dir_end(szDest);
							lstrcatA(szDest, lpFilePartSrc);
						}

						// (iv) Source and dest must be on same drive
						if(CharUpperA((LPTSTR)MAKELONG(src[0], 0)) != CharUpperA((LPTSTR)MAKELONG(dest[0], 0)))
						{
							fprintf(stderr, "The source - \"%s\" - and destination - \"%s\" - are on different drives\n\n", src, dest);

							bShowUsage = true;
						}
					}
				}
			}
		}

		/* Now carry out the operation */
		if(!bShowUsage)
		{
			switch(operation)
			{
				default:
					stlsoft_message_assert("unrecognised operation type", 0);
				case	Operation::usage:
					bShowUsage = true;
					break;
				case	Operation::list:
					bShowUsage = !show_list();
					break;
				case	Operation::edit:
					bShowUsage = !edit_item(static_cast<size_t>(from), src, dest);
					break;
				case	Operation::find:
					bShowUsage = !find_item(src);
					break;
				case	Operation::addition:
					bShowUsage = !add_item(src, dest);
					break;
				case	Operation::deletion:
					bShowUsage = !delete_item(static_cast<size_t>(from));
					break;
				case	Operation::movement:
					bShowUsage = !move_item(from, to);
					break;
			}
		}

		if(bShowUsage)
		{
			usage(1, NULL);
		}
	}

    return iRet;
}

////////////////////////////////////////////////////////////////////////////////

void load_entries(container_t &list)
{
	list.Load();
}

void save_entries(const container_t &list)
{
	list.Store();
}

////////////////////////////////////////////////////////////////////////////////

void usage(int bExit, char const *reason)
{
	FILE	*stm	=	(NULL == reason) ? stdout : stderr;

    fprintf(stm, "\n");
	fprintf(stm, "  %s %s Tool, v%d.%d.%d.%04d\n", COMPANY_NAME, TOOL_NAME, _mccVerHi, _mccVerLo, _mccVerRev, _mccBldNum);
    fprintf(stm, "\n");

	if(NULL != reason)
	{
		fprintf(stm, "  Error: %s\n", reason);
		fprintf(stm, "\n");
	}

	fprintf(stm, "  USAGE 1: %s [-{a | d | e | f | l | m}] [<from>] [<to>] [<src>] [<dest>]\n"
					"\n"
					"    a   -   add an item. Requires <src>. If <dest> is missing, <src> is \n"
					"            marked for deletion\n"
					"    d   -   delete an item. Requires <from>\n"
					"    e   -   edit/replace an item. Requires <from> <src> <dest> \n"
					"    f   -   finds an item in the current pending operations. Requires <src>\n"
					"    l   -   list the current operations. (Default) \n"
					"    m   -   move an item. Requires <from> <to> \n"
					"\n"
					"    <from>  -   an item index (number), eg. 3\n"
					"    <to>    -   an item index\n"
					"    <src>   -   an item name (full source path, may have quotes), eg.\n"
					"                         \"c:\\program files\\x.txt\"\n"
					"    <dest>  -   an item name (may be blank, in which case <src> is deleted)\n"
					"\n", EXE_NAME);
    fprintf(stm, "  USAGE 2: %s -?\n", EXE_NAME);
    fprintf(stm, "\n");
    fprintf(stm, "    Displays this help\n");
    fprintf(stm, "\n");
	fprintf(stm, "  Contact %s\n", _nameSynesisSoftware);
	fprintf(stm, "    at \"%s\",\n", _wwwSynesisSoftware_SystemTools);
	fprintf(stm, "    or, via email, at \"%s\"\n", _emailSynesisSoftware_SystemTools);
    fprintf(stm, "\n");

	if(bExit)
	{
		exit(EXIT_FAILURE);
	}
}

bool show_list(const int highlightIndex)
{
	/* Now walk through the list. */

	static const char	*formats[4] =
	{
			"%d: %s%s\n"
			"  =>  %s\n"
		,	"%d: \"%s\"%s\n"
			"  =>  %s\n"
		,	"%d: %s\n"
			"  =>  \"%s%s\"\n"
		,	"%d: \"%s\"%s\n"
			"  =>  \"%s\"\n"
	};

	container_t	list;
	int			i;

	load_entries(list);

	container_t::const_iterator	begin	=	list.begin();
	container_t::const_iterator	end		=	list.end();

	for(i = 0; begin != end; ++begin, ++i)
	{
		const container_t::value_type			&value		=	*begin;
		int										index		=	0;
		char									sz[2048];
		char const						*const	highlight	=	(highlightIndex == i) ? " - Found Item!" : "";

		if(	value.first.length() == 0 ||
			value.first.length() > 73 ||
#ifdef __DMC__
			value.first.find(' ') != string_t::npos)
#else /* ? __DMC__ */
			value.first.find(' ') != container_t::value_type::first_type::npos)
#endif /* __DMC__ */
		{
			index += 1;
		}

		if(	value.second.length() == 0 ||
			value.second.length() > 73 ||
#ifdef __DMC__
			value.second.find(' ') != string_t::npos)
#else /* ? __DMC__ */
			value.second.find(' ') != container_t::value_type::second_type::npos)
#endif /* __DMC__ */
		{
			index += 2;
		}



		sprintf(sz, formats[index], i, value.first.c_str(), highlight, value.second.c_str());

		fprintf(stderr, sz);
		OutputDebugStringA(sz);
	}

	return true;
}

bool add_item(const char *src, const char *dest)
{
	container_t	list;

	if(NULL == dest)
	{
		dest = "";
	}

	return	list.Load() >= 0 &&
			list.Push(src, dest) >= 0 &&
			list.Store() >= 0;
}

bool find_item(char const *src)
{
	container_t	list;

	if(list.Load() < 0)
	{
		return false;
	}

	int	index	=	list.Find(src);

	if(index >= 0)
	{
		show_list(index);

		return true;
	}

	return false;
}

bool delete_item(size_t from)
{
	container_t	list;

	return	list.Load() >= 0 &&
			list.Delete(from) >= 0 &&
			list.Store() >= 0;
}

bool edit_item(size_t from, const char *src, const char *dest)
{
	container_t	list;

	return	list.Load() >= 0 &&
			list.Set(from, src, dest) >= 0 &&
			list.Store() >= 0;
}

bool move_item(int from, int to)
{
	container_t	list;

	if(list.Load() < 0)
	{
		return false;
	}

	if(	from < 0 ||
		to < 0 ||
		from == to ||
		from >= static_cast<int>(list.size()) ||
		to >= static_cast<int>(list.size()))
	{
		return false;
	}

	if(to < from)
	{
		std::swap(to, from);
	}

	entry_t	toValue	=	list[static_cast<size_t>(to)];

	if(list.Insert(static_cast<size_t>(from), toValue.first, toValue.second) < 0)
	{
		return false;
	}

	if(to > from)
	{
		++to;
	}

	return	list.Delete(static_cast<size_t>(to)) >= 0 &&
			list.Store() >= 0;
}

////////////////////////////////////////////////////////////////////////////////
