/* /////////////////////////////////////////////////////////////////////////////
 * File:        udp_tx.cpp
 *
 * Purpose:     Implementation file for the udp_tx project.
 *
 * Created:     20th September 2005
 * Updated:     20th September 2005
 *
 * Status:      Wizard-generated
 *
 * License:     (Licensed under the Synesis Software Open License)
 *
 *              Copyright 1999-2005, Synesis Software Pty Ltd.
 *              All rights reserved.
 *
 *              www:        http://www.synesis.com.au/software
 *
 *              email:      software@synesis.com.au
 *
 *              This source code is placed into the public domain 2005
 *              by Synesis Software Pty Ltd. There are no restrictions
 *              whatsoever to your use of the software. 
 *
 *              This source code is provided by Synesis Software Pty Ltd "as is"
 *              and any warranties, whether expressed or implied, including, but
 *              not limited to, the implied warranties of merchantability and
 *              fitness for a particular purpose are disclaimed. In no event
 *              shall the Synesis Software Pty Ltd be liable for any direct,
 *              indirect, incidental, special, exemplary, or consequential
 *              damages (including, but not limited to, procurement of
 *              substitute goods or services; loss of use, data, or profits; or
 *              business interruption) however caused and on any theory of
 *              liability, whether in contract, strict liability, or tort
 *              (including negligence or otherwise) arising in any way out of
 *              the use of this software, even if advised of the possibility of
 *              such damage. 
 *
 *              Neither the name of Synesis Software Pty Ltd nor the names of
 *              any subdivisions, employees or agents of Synesis Software Pty
 *              Ltd, nor the names of any other contributors to this software
 *              may be used to endorse or promote products derived from this
 *              software without specific prior written permission. 
 *
 * ////////////////////////////////////////////////////////////////////////// */

/* STLSoft Header Files */
#include <stlsoft.h>
#include <stlsoft/simple_string.hpp>

/* Pantheios Header Files */
#include <pantheios/pantheios.hpp>
#include <pantheios/implicit_link/core.h>
#include <pantheios/implicit_link/fe_simple.h>
#include <pantheios/implicit_link/be_fprintf.h>

/* PlatformSTL Header Files */
#include <platformstl/platformstl.h>

#if defined(PLATFORMSTL_OS_IS_UNIX)

# include <unixstl.h>

# ifdef _WIN32
#  include <unixem/implicit_link.h>
# endif /* _WIN32 */

#elif defined(PLATFORMSTL_OS_IS_WIN32)

# include <winstl.h>
# include <winstl/error_desc.hpp>
# include <winstl/exceptions.hpp>
# include <winstl/file_functions.hpp>

#else /* ? operating system */

# error Operating system not discriminated

#endif /* operating system */


/* InetSTL Header Files */
#if defined(PLATFORMSTL_OS_IS_WIN32)
//# include <inetstl.h>
//# include <inetstl/connection.hpp>
#endif /* operating system */


/* Standard C++ Header Files */
#include <exception>

#if !defined(__WATCOMC__) && \
    (   !defined(_MSC_VER) || \
        _MSC_VER >= 1100)

#else /* ? __WATCOMC__ */
namespace std
{
    using ::exception;
}
#endif /* __WATCOMC__ */

/* Standard C Header Files */
#include <stdio.h>
#include <stdlib.h>

/* Miscellaneous includes */

#ifdef _MSC_VER
# include <crtdbg.h>
#endif /* _MSC_VER */

/* /////////////////////////////////////////////////////////////////////////////
 * Version information
 *
 * Normally, Synesis Software components & tools are "versioned" by making use
 * of tool-generated header files, one of which - MBldHdr.h - defines symbolic
 * constants representing major, minor, revision and build numbers.
 *
 * Since this file is to be available in a more public forum, these constants 
 * are defined here, rather than encumber any developers who may wish to compile
 * it with a lot of Synesis-specific gunk.
 */

#ifdef _SYNSOFT_INTERNAL_BUILD
# include "MLCompId.h"
#else /* ? _SYNSOFT_INTERNAL_BUILD */
# define _nameSynesisSoftware               "Synesis Software (Pty) Ltd"
# define _wwwSynesisSoftware_SystemTools    "http://synesis.com.au/systools.html"
# define _emailSynesisSoftware_SystemTools  "software@synesis.com.au"
#endif /* _SYNSOFT_INTERNAL_BUILD */

#ifdef _SYNSOFT_INTERNAL_BUILD
# include "MBldHdr.h"
# include "showver.h"

extern "C" const int  _mccVerHi   =   __SYV_MAJOR;
extern "C" const int  _mccVerLo   =   __SYV_MINOR;
extern "C" const int  _mccVerRev  =   __SYV_REVISION;
extern "C" const int  _mccBldNum  =   __SYV_BUILDNUMBER;

#else /* ? _SYNSOFT_INTERNAL_BUILD */
extern "C" const int  _mccVerHi   =   1;
extern "C" const int  _mccVerLo   =   0;
extern "C" const int  _mccVerRev  =   1;
extern "C" const int  _mccBldNum  =   1;
#endif /* _SYNSOFT_INTERNAL_BUILD */

# define    COMPANY_NAME    "Synesis Software"
# define    TOOL_NAME       "UDP Transmitter"
# define    EXE_NAME        "udp_tx"

extern "C" char const FE_SIMPLE_PROCESS_IDENTITY[]	=	EXE_NAME;

/* /////////////////////////////////////////////////////////////////////////////
 * Forward declarations
 */

static void usage(int bExit, char const *reason, int invalidArg, int argc, char *argv[]);
static int do_tx(unsigned port, char const *message, int bMessageFile, unsigned interval);

/* ////////////////////////////////////////////////////////////////////////// */

static int main_(int argc, char **argv)
{
	char const	*message		=	NULL;
	char const	*portStr		=	NULL;
	int			bMessageFile	=	false;
	unsigned	interval		=	0;
	unsigned	port			=	0;

    for(int i = 1; i < argc; ++i)
    {
        char const  *arg    =   argv[i];

        if(arg[0] == '-')
        {
			if(arg[1] == '-')
			{
				usage(1, "Invalid argument(s) specified", i, argc, argv);
            }
            else
			{
				switch(arg[1])
				{
                    case    '?':
                        usage(1, NULL, -1, argc, argv);
                        break;
					case	'f':
						bMessageFile = true;
						break;
					case	'i':
						interval = ::strtoul(arg + 2, NULL, 0);
						if(0 == interval)
						{
							usage(1, "Invalid interval specified; must be greater than 0", i, argc, argv);
						}
						break;
                    default:
                        usage(1, "Invalid argument(s) specified", i, argc, argv);
                        break;
                }
            }
        }
        else
        {
            if(message == NULL)
            {
                message = arg;
            }
            else if(portStr == NULL)
            {
                portStr = arg;
            }
            else
            {
                usage(1, "<root-paths> and <search-spec> already specified", i, argc, argv);
            }
        }
    }

	if(NULL == message)
	{
		usage(1, "Must specify message", -1, argc, argv);
	}

	if(NULL == portStr)
	{
		portStr = "48001";
	}

	port = ::strtoul(portStr, NULL, 0);
	if(0 == port)
	{
		usage(1, "Must specify port (as a number between 1 and 65535)", -1, argc, argv);
	}

	if(0 == interval)
	{
		interval = 10;
	}

#if defined(PLATFORMSTL_OS_IS_WIN32)
	WSADATA	data;

	if(0 != ::WSAStartup(0x0202, &data))
	{
		pantheios::log_CRITICAL("Cannot initialise WinSock libraries: ", winstl::error_desc());

		return EXIT_FAILURE;
	}
#endif /* operating system */


	return do_tx(port, message, bMessageFile, interval);
}

int main(int argc, char *argv[])
{
	int	res;

#if 0
	{ for(size_t i = 0; i < 0xffffffff; ++i){} }
#endif /* 0 */

#if defined(_MSC_VER) && \
    defined(_DEBUG)
    _CrtMemState		memState;
#endif /* _MSC_VER && _MSC_VER */

#if defined(_MSC_VER) && \
    defined(_DEBUG)
    _CrtMemCheckpoint(&memState);
#endif /* _MSC_VER && _MSC_VER */

	try
	{
#if defined(_DEBUG) || \
    defined(__SYNSOFT_DBS_DEBUG)
		puts("udp_tx: " __STLSOFT_COMPILER_LABEL_STRING);
#endif /* debug */

		res = main_(argc, argv);
	}
	catch(std::exception &x)
	{
		fprintf(stderr, "Unhandled error: %s\n", x.what());

		res = EXIT_FAILURE;
	}
	catch(...)
	{
		fprintf(stderr, "Unhandled unknown error\n");

		res = EXIT_FAILURE;
	}

#if defined(_MSC_VER) && \
    defined(_DEBUG)
    _CrtMemDumpAllObjectsSince(&memState);
#endif /* _MSC_VER && _MSC_VER */

	return res;
}

/* /////////////////////////////////////////////////////////////////////////////
 * Transmission
 */

static int do_tx(unsigned port, char const *message, int bMessageFile, unsigned interval)
{
	stlsoft::simple_string	message_;
	size_t					msgLen;

	if(bMessageFile)
	{
		if(0 == winstl::load_text_file(message, message_))
		{
			pantheios::log_ERROR(message, " cannot be opened: ", message, winstl::error_desc());

			return EXIT_FAILURE;
		}
		else
		{
			message =	message_.c_str();
			msgLen	=	message_.size();
		}
	}
	else
	{
		msgLen	=	::strlen(message);
	}

	SOCKET	sk	=	::socket(AF_INET, SOCK_DGRAM, IPPROTO_UDP);

	if(SOCKET_ERROR == sk)
	{
		pantheios::log_ERROR("Could not create send socket: ", winstl::error_desc());

//		throw winstl::windows_exception("Could not create send socket", ::GetLastError());
		return EXIT_FAILURE;
	}
	else
	{
		struct sockaddr_in sin;

		sin.sin_family		=	AF_INET;
		sin.sin_port		=	htons(port);
		sin.sin_addr.s_addr =	inet_addr("127.0.0.1");

		for(;; ::Sleep(interval * 1000))
		{
			int r	=	sendto(sk, message, msgLen, 0, (struct sockaddr const *)&sin, sizeof(sin));

			if(SOCKET_ERROR == r)
			{
				pantheios::log_ERROR("Could not send message: ", winstl::error_desc());

				break;
			}
		}
	}

#if defined(PLATFORMSTL_OS_IS_WIN32)
//# include <inetstl.h>
//# include <inetstl/connection.hpp>
#endif /* operating system */

	return EXIT_SUCCESS;
}

/* /////////////////////////////////////////////////////////////////////////////
 * usage
 */

static void usage(int bExit, char const *reason, int invalidArg, int argc, char *argv[])
{
	FILE	*stm	=	(NULL == reason) ? stdout : stderr;

    fprintf(stm, "\n");
    fprintf(stm, "  %s %s Tool, v%d.%d.%d.%04d\n", COMPANY_NAME, TOOL_NAME, _mccVerHi, _mccVerLo, _mccVerRev, _mccBldNum);
    fprintf(stm, "\n");

    if(NULL != reason)
    {
        fprintf(stm, "  Error: %s\n", reason);
        fprintf(stm, "\n");
    }

	if(0 < invalidArg)
	{
		fprintf(stm, "  First invalid argument #%d: %s\n", invalidArg, argv[invalidArg]);
		fprintf(stm, "  Arguments were (first bad marked *):\n\n");
		for(int i = 1; i < argc; ++i)
		{
			fprintf(stm, "  %s%s\n", (i == invalidArg) ? "* " : "  ", argv[i]);
		}
        fprintf(stm, "\n");
	}

    fprintf(stm, "  USAGE 1: %s [-f] [-i<interval>] <message> [<port>]\n", EXE_NAME);
    fprintf(stm, "    where:\n\n");
    fprintf(stm, "    -f         - treat <message> as a text file, containing message to send\n");
    fprintf(stm, "    -i         - set repeat interval to <interval> seconds; defaults to 10\n");
    fprintf(stm, "    <message>  - message to send on UDP broadcast\n");
    fprintf(stm, "    <port>     - port on which to broadcast; defaults to 48001\n");
    fprintf(stm, "\n");
    fprintf(stm, "  USAGE 2: %s -?\n", EXE_NAME);
    fprintf(stm, "\n");
    fprintf(stm, "    Displays this help\n");
    fprintf(stm, "\n");
    fprintf(stm, "  Contact %s\n", _nameSynesisSoftware);
    fprintf(stm, "    at \"%s\",\n", _wwwSynesisSoftware_SystemTools);
    fprintf(stm, "    or, via email, at \"%s\"\n", _emailSynesisSoftware_SystemTools);
    fprintf(stm, "\n");

    if(bExit)
    {
        exit(EXIT_FAILURE);
    }
}

/* ////////////////////////////////////////////////////////////////////////// */
