// MainFrm.cpp : implmentation of the CMainFrame class
//
// Updated: 17th September 2005
//
/////////////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "resource.h"

#include "aboutdlg.h"
#include "ws_trimView.h"
#include "MainFrm.h"

BOOL CMainFrame::PreTranslateMessage(MSG* pMsg)
{
	if(parent_class_type::PreTranslateMessage(pMsg))
	{
		return TRUE;
	}

	return m_view.PreTranslateMessage(pMsg);
}

BOOL CMainFrame::OnIdle()
{
	UIUpdateToolBar();
	return FALSE;
}

/* ////////////////////////////////////////////////////////////////////////// */

struct lvitem_has_data
{
public:
	lvitem_has_data(HWND	hwnd
				,	LPARAM	param)
		: m_param(param)
	{}

public:
	template <typename I>
	bool operator ()(I const &item) const
	{
		return m_param == item.data();
	}

private:
	const LPARAM	m_param;
};

struct lvitem0_has_pid
{
public:
	lvitem0_has_pid(HWND	hwnd
				,	LPARAM	param)
		: m_hwnd(hwnd)
		, m_param(param)
	{}

public:
	template <typename I>
	bool operator ()(I const &item) const
	{
		TCHAR	sz1[101];

		ListView_GetItemText(m_hwnd, item.index(), 0, &sz1[0], STLSOFT_NUM_ELEMENTS(sz1));

		int		pid	=	atoi(sz1);

		return m_param == pid;
	}

private:
	HWND			m_hwnd;
	const LPARAM	m_param;
};

struct wci_select
{
public:
	template <typename I>
	void operator ()(I &item) const
	{
#ifdef _DEBUG
#else /* ? _DEBUG */
# error Is this read-only?
#endif /* _DEBUG */
		ListView_SetItemState(item.hwnd(), item.index(), LVIS_SELECTED, LVIS_SELECTED);
	}
};

/* /////////////////////////////////////////////////////////////////////////////
 * Forward declarations
 */

LRESULT CMainFrame::OnCreate(UINT /*uMsg*/, WPARAM /*wParam*/, LPARAM /*lParam*/, BOOL& /*bHandled*/)
{
	CreateSimpleToolBar();

	CreateSimpleStatusBar();

	m_hWndClient = m_view.Create(m_hWnd, rcDefault, NULL, WS_CHILD | WS_VISIBLE | WS_CLIPSIBLINGS | WS_CLIPCHILDREN | LVS_REPORT | LVS_SHOWSELALWAYS, WS_EX_CLIENTEDGE);

	UIAddToolBar(m_hWndToolBar);
	UISetCheck(ID_VIEW_TOOLBAR, 1);
	UISetCheck(ID_VIEW_STATUS_BAR, 1);

	// register object for message filtering and idle updates
	CMessageLoop* pLoop = _Module.GetMessageLoop();
	ATLASSERT(pLoop != NULL);
	pLoop->AddMessageFilter(this);
	pLoop->AddIdleHandler(this);

	// Setup the view

	MListView_SetMListViewFlags(m_view
							,	MLVF_COLUMNSORT | MLVF_TYPESORT | MLVF_VALUESORT | MLVF_IGNORECASEINSORT | MLVF_NORESETCOLONINSERT
							,	MLVF_COLUMNSORT | MLVF_TYPESORT | MLVF_VALUESORT | MLVF_IGNORECASEINSORT | MLVF_NORESETCOLONINSERT);

	ListView_SetExtendedListViewStyleEx(m_view
									,	LVS_EX_FULLROWSELECT
									,	LVS_EX_FULLROWSELECT);

	m_view.InsertColumn(0, "PID", LVCFMT_LEFT, 100, 0);
	m_view.InsertColumn(1, "Process Name", LVCFMT_LEFT, 150, 1);
	m_view.InsertColumn(2, "Path", LVCFMT_LEFT, 400, 2);
	m_view.InsertColumn(3, "Working Set", LVCFMT_LEFT, 100, 3);
	m_view.InsertColumn(4, "Peak Working Set", LVCFMT_LEFT, 125, 4);

	// Status bar
	
#if 0
	1. Working set total
	2. Number of processes
	3. WS delta
#endif /* 0 */

	// Populate the view the first time
	RefreshProcessList();

	return 0;
}

void CMainFrame::RefreshProcessList()
{
	winstl::pid_sequence	pids;

	// The algorithm is as follows:

	// 1. Iterate through each element in the pid sequence.
	//  1.a If the element exists in the view, then update the 
	//       working set information and invalidate those cells
	//  1.b If the element does not exist in the view, then add
	//       anew
	// 2. Iterate through each element in the view, and remove
	//       those that are complete
	//
	// There is a possible race condition, whereby a process would
	// complete and a new one with the same pid would enter. This
	// should be dealt with later


	// 1. Add, or refresh, entries that are 
	{ for(winstl::pid_sequence::const_iterator b = pids.begin(); b != pids.end(); ++b)
	{
		stlsoft::scoped_handle<HANDLE>				hProcess(::OpenProcess(PROCESS_QUERY_INFORMATION | PROCESS_VM_READ, false, *b), ::CloseHandle);
		winstl::listview_sequence					lvs(m_view);
		winstl::listview_sequence::const_iterator	it	=	std::find_if(lvs.begin(), lvs.end(), lvitem0_has_pid(m_view, *b));

		if(it != lvs.end())
		{
			winstl::listview_sequence::value_type	v	=	*it;
			PROCESS_MEMORY_COUNTERS					pmc;

			if(::GetProcessMemoryInfo(hProcess.handle(), &pmc, sizeof(pmc)))
			{
				SynesisCstCtrls::CellItem	item;

				item.mask		=	MCCIF_TYPE;
				item.nItem		=	v.index();
				item.nSubItem	=	3;

				MListView_GetItem(m_view, &item);

				if(item.lVal != pmc.WorkingSetSize)
				{
					item.type		=	MCCIT_LONG | MCCIT_LONG_AS_FILESIZE;

					item.nSubItem	=	3;
					item.lVal		=	pmc.WorkingSetSize;

					MListView_SetItem(m_view, &item);
				}

				item.nSubItem	=	4;

				MListView_GetItem(m_view, &item);

				if(item.lVal != pmc.PeakWorkingSetSize)
				{
					item.type		=	MCCIT_LONG | MCCIT_LONG_AS_FILESIZE;

					item.nSubItem	=	4;
					item.lVal		=	pmc.PeakWorkingSetSize;

					MListView_SetItem(m_view, &item);
				}
			}
		}
		else
		{
			if(!hProcess.empty())
			{
				char					sz[101];
				int						iItem;
				winstl::path			path(getModulePath(hProcess.handle(), NULL));
				PROCESS_MEMORY_COUNTERS	pmc;

				// 0. PID
				iItem = SynesisCstCtrls::SyListView_InsertLongItem(m_view, -1, *b);

				if(iItem >= 0)
				{
					// 1. Process Name
					m_view.AddItem(iItem, 1, path.get_file());

					// 2. Path

					m_view.AddItem(iItem, 2, stlsoft::c_str_ptr(path));

					// 3 & 4. Working set & Max Working Set
					if(::GetProcessMemoryInfo(hProcess.handle(), &pmc, sizeof(pmc)))
					{
						SynesisCstCtrls::CellItem	item;

						item.mask		=	MCCIF_TYPE;
						item.nItem		=	iItem;
						item.type		=	MCCIT_LONG | MCCIT_LONG_AS_FILESIZE;

						item.nSubItem	=	3;
						item.lVal		=	pmc.WorkingSetSize;

						MListView_SetItem(m_view, &item);

						item.nSubItem	=	4;
						item.lVal		=	pmc.PeakWorkingSetSize;

						MListView_SetItem(m_view, &item);
					}

					// Item data
					MListView_SetItemData(m_view, iItem, hProcess.handle());

					hProcess.detach();
				}
			}
		}
	}}

	// 2. Remove any entries from the view that are not in the process list
	{ winstl::listview_sequence	lvs(m_view);
	for(winstl::listview_sequence::const_iterator b = lvs.begin(); b != lvs.end(); ++b)
	{
		winstl::listview_sequence::value_type	v			=	*b;
		HANDLE									hProcess	=	(HANDLE)v.data();

		if(WAIT_OBJECT_0 == ::WaitForSingleObject(hProcess, 0))
		{
			ATLTRACE("process complete");
		}
	}}
}

LRESULT CMainFrame::OnFileExit(WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/, BOOL& /*bHandled*/)
{
	PostMessage(WM_CLOSE);

	return 0;
}

LRESULT CMainFrame::OnTrim(WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/, BOOL& /*bHandled*/)
{
	stlsoft::auto_buffer<int>	selections(SynesisCstCtrls::ListView_GetSelItems(m_view, 0, NULL));
	int							n	=	SynesisCstCtrls::ListView_GetSelItems(m_view, selections.size(), &selections[0]);

	for(int i = 0; i < n; ++i)
	{
		HANDLE	hProcess	=	(HANDLE)SynesisCstCtrls::ListView_GetItemData(m_view, selections[i]);
		HANDLE	hProcess2;

#if 0
		if(::DuplicateHandle(::GetCurrentProcess(), hProcess, ::GetCurrentProcess(), &hProcess2, PROCESS_SET_INFORMATION | PROCESS_QUERY_INFORMATION, false, 0))
#else /* ? 0 */
		if(::DuplicateHandle(::GetCurrentProcess(), hProcess, ::GetCurrentProcess(), &hProcess2, PROCESS_ALL_ACCESS, false, 0))
#endif /* 0 */
		{
			if(!::EmptyWorkingSet(hProcess2))
			{
				ATLTRACE("Failed to empty: %d\n", ::GetLastError());
			}

			::CloseHandle(hProcess2);
		}
	}

	return 0;
}

LRESULT CMainFrame::OnRefresh(WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/, BOOL& /*bHandled*/)
{
	RefreshProcessList();

	return 0;
}

LRESULT CMainFrame::OnSelectAll(WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/, BOOL& /*bHandled*/)
{
	winstl::listview_sequence	lvs(m_view);

	std::for_each(lvs.begin(), lvs.end(), wci_select());

	return 0;
}

LRESULT CMainFrame::OnFileNew(WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/, BOOL& /*bHandled*/)
{
	// TODO: add code to initialize document

	return 0;
}

LRESULT CMainFrame::OnViewToolBar(WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/, BOOL& /*bHandled*/)
{
	BOOL bVisible = !::IsWindowVisible(m_hWndToolBar);
	::ShowWindow(m_hWndToolBar, bVisible ? SW_SHOWNOACTIVATE : SW_HIDE);
	UISetCheck(ID_VIEW_TOOLBAR, bVisible);
	UpdateLayout();
	return 0;
}

LRESULT CMainFrame::OnViewStatusBar(WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/, BOOL& /*bHandled*/)
{
	BOOL bVisible = !::IsWindowVisible(m_hWndStatusBar);
	::ShowWindow(m_hWndStatusBar, bVisible ? SW_SHOWNOACTIVATE : SW_HIDE);
	UISetCheck(ID_VIEW_STATUS_BAR, bVisible);
	UpdateLayout();
	return 0;
}

LRESULT CMainFrame::OnAppAbout(WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/, BOOL& /*bHandled*/)
{
	::atlstl::SyAboutDialogId<IDD_ABOUTBOX>	dlg;

	dlg.DoModal();

	return 0;
}

/* ////////////////////////////////////////////////////////////////////////// */

#if 0
				{
					winstl::process_module_sequence	modules(hProcess.handle());

					for(winstl::process_module_sequence::const_iterator b = modules.begin(); b != modules.end(); ++b)
					{
	//					::GetModuleFileNameEx(hProcess, NULL, &szFileName[0], STLSOFT_NUM_ELEMENTS(szFileName));

	//					m_view.AddItem(iItem, 1, sz);
					}
				}
#endif /* 0 */

/* ////////////////////////////////////////////////////////////////////////// */

