/* /////////////////////////////////////////////////////////////////////////////
 * File:        safearray_sequence_test.cpp
 *
 * Purpose:     Implementation file for the safearray_sequence_test project.
 *
 * Created:     17th April 2004
 * Updated:     21st November 2004
 *
 * Status:      Wizard-generated
 *
 * License:     (Licensed under the Synesis Software Open License)
 *
 *              Copyright (C) 1999-2004, Synesis Software Pty Ltd.
 *              All rights reserved.
 *
 *              www:        http://www.synesis.com.au/software
 *
 *              email:      software@synesis.com.au
 *
 *              This source code is placed into the public domain 2004
 *              by Synesis Software Pty Ltd. There are no restrictions
 *              whatsoever to your use of the software.
 *
 *              This source code is provided by Synesis Software Pty Ltd "as is"
 *              and any warranties, whether expressed or implied, including, but
 *              not limited to, the implied warranties of merchantability and
 *              fitness for a particular purpose are disclaimed. In no event
 *              shall the Synesis Software Pty Ltd be liable for any direct,
 *              indirect, incidental, special, exemplary, or consequential
 *              damages (including, but not limited to, procurement of
 *              substitute goods or services; loss of use, data, or profits; or
 *              business interruption) however caused and on any theory of
 *              liability, whether in contract, strict liability, or tort
 *              (including negligence or otherwise) arising in any way out of
 *              the use of this software, even if advised of the possibility of
 *              such damage.
 *
 *              Neither the name of Synesis Software Pty Ltd nor the names of
 *              any subdivisions, employees or agents of Synesis Software Pty
 *              Ltd, nor the names of any other contributors to this software
 *              may be used to endorse or promote products derived from this
 *              software without specific prior written permission.
 *
 * ////////////////////////////////////////////////////////////////////////// */

#include <stdio.h>
#include <stdlib.h>

#include <stlsoft.h>
#include <stlsoft_integer_to_string.h>

#include <comstl.h>
#include <comstl_safearray_sequence.h>

#include <algorithm>
#include <exception>

/* ////////////////////////////////////////////////////////////////////////// */

void usage(int bExit);

#if (   defined(WIN32) || \
        defined(_WIN32)) && \
    !defined(_INC_WINDOWS)
extern "C" void __stdcall Sleep(unsigned long);
#endif /* WIN32 */

/* ////////////////////////////////////////////////////////////////////////// */

void print_long(long const &l)
{
	printf("%ld ", l);
}

void print_var(VARIANT const &v)
{
	switch(v.vt)
	{
		case	VT_I4:
			printf("%ld ", v.lVal);
			break;
	}
}

void print_bstr(BSTR const &bstr)
{
	wprintf(L"%s ", bstr);
}

void test_long(size_t d0, size_t d1)
{
	SAFEARRAYBOUND	bounds[2] =
	{
			{	d0,		0	}
		,	{	d1,		0	}
	};
	LPSAFEARRAY		psa	=	::SafeArrayCreate(VT_I4, stlsoft_num_elements(bounds), bounds);
	void			*pv;
	HRESULT			hr	=	::SafeArrayAccessData(psa, &pv);

	long			*pl	=	static_cast<long*>(pv);
	size_t			l;

	for(l = 0; l < bounds[0].cElements * bounds[1].cElements; ++l)
	{
		pl[l] = l;
	}
	::SafeArrayUnaccessData(psa);

	comstl::safearray_sequence<long>	array(psa);

	puts("long:");
	std::for_each(array.begin(), array.end(), print_long);
	puts("");

	::SafeArrayDestroy(psa);
}

void test_VARIANT_long(size_t d0, size_t d1)
{
	SAFEARRAYBOUND	bounds[2] =
	{
			{	d0,		0	}
		,	{	d1,		0	}
	};
	LPSAFEARRAY		psa		=	::SafeArrayCreate(VT_VARIANT /* | VT_I4 */, stlsoft_num_elements(bounds), bounds);
	void			*pv;
	HRESULT			hr		=	::SafeArrayAccessData(psa, &pv);

	VARIANT			*pvar	=	static_cast<VARIANT*>(pv);
	size_t			l;

	for(l = 0; l < bounds[0].cElements * bounds[1].cElements; ++l)
	{
		pvar[l].vt		=	VT_I4;
		pvar[l].lVal	=	l;
	}
	::SafeArrayUnaccessData(psa);

	comstl::safearray_sequence<VARIANT>	array(psa);

	puts("VARIANT:");
	std::for_each(array.begin(), array.end(), print_var);
	puts("");

	::SafeArrayDestroy(psa);
}

void test_BSTR(size_t d0, size_t d1)
{
	SAFEARRAYBOUND	bounds[2] =
	{
			{	d0,		0	}
		,	{	d1,		0	}
	};
	LPSAFEARRAY		psa		=	::SafeArrayCreate(VT_BSTR, stlsoft_num_elements(bounds), bounds);
	void			*pv;
	HRESULT			hr		=	::SafeArrayAccessData(psa, &pv);

	BSTR			*pbstr	=	static_cast<BSTR*>(pv);
	size_t			l;

	for(l = 0; l < bounds[0].cElements * bounds[1].cElements; ++l)
	{
		wchar_t		sz[21];
		wchar_t		szNum[41];

		wsprintfW(szNum, L"[%s] ", stlsoft::integer_to_string(sz, stlsoft_num_elements(sz), static_cast<stlsoft::sint32_t>(l)));

		pbstr[l]	=	::SysAllocString(szNum);
	}
	::SafeArrayUnaccessData(psa);

	comstl::safearray_sequence<BSTR>	array(psa);

	puts("BSTR:");
	std::for_each(array.begin(), array.end(), print_bstr);
	puts("");

	::SafeArrayDestroy(psa);
}

int main(int argc, char *argv[])
{
#if 0
	::Sleep(100000);
#endif /* 0 */

    int									iRet    =   0;
    int									i;

	puts("safearray_sequence_test: " __STLSOFT_COMPILER_LABEL_STRING);

	for(i = 1; i < argc; ++i)
	{
		const char	*arg	=	argv[i];

		if(arg[0] == '-')
		{
			if(arg[1] == '-')
			{
				/* -- arguments */
			}
			else
			{
				/* - arguments */
			}
		}
		else
		{
			/* other arguments */
		}
	}

	/* . */
	test_long(10, 5);
	test_VARIANT_long(2, 4);
	test_BSTR(2, 4);

    return iRet;
}

/* ////////////////////////////////////////////////////////////////////////// */

void usage(int bExit)
{
    fprintf(stderr, "\n");
    fprintf(stderr, "  safearray_sequence_test\n\n");

#if 0
    fprintf(stderr, "  USAGE: safearray_sequence_test [{-w | -p<root-paths> | -h}] [-u] [-d] [{-v | -s}] <search-spec>\n");
    fprintf(stderr, "    where:\n\n");
    fprintf(stderr, "    -w             - searches from the current working directory. The default\n");
    fprintf(stderr, "    -p<root-paths> - searches from the given root path(s), separated by \';\',\n");
    fprintf(stderr, "                     eg.\n");
    fprintf(stderr, "                       -p\"c:\\windows;x:\\bin\"\n");
    fprintf(stderr, "    -r             - deletes readonly files\n");
    fprintf(stderr, "    -h             - searches from the roots of all drives on the system\n");
    fprintf(stderr, "    -d             - displays the path(s) searched\n");
    fprintf(stderr, "    -u             - do not act recursively\n");
    fprintf(stderr, "    -v             - verbose output. Prints time, attributes, size and path. (default)\n");
    fprintf(stderr, "    -s             - succinct output. Prints path only\n");
    fprintf(stderr, "    <search-spec>  - one or more file search specifications, separated by \';\',\n");
    fprintf(stderr, "                     eg.\n");
    fprintf(stderr, "                       \"*.exe\"\n");
    fprintf(stderr, "                       \"myfile.ext\"\n");
    fprintf(stderr, "                       \"*.exe;*.dll\"\n");
    fprintf(stderr, "                       \"*.xl?;report.*\"\n");
    fprintf(stderr, "\n");
	fprintf(stderr, "  Contact %s\n", _nameSynesisSoftware);
	fprintf(stderr, "    at \"%s\",\n", _wwwSynesisSoftware_SystemTools);
	fprintf(stderr, "    or, via email, at \"%s\"\n", _emailSynesisSoftware_SystemTools);
    fprintf(stderr, "\n");
#endif /* 0 */

    if(bExit)
    {
        exit(1);
    }
}

/* ////////////////////////////////////////////////////////////////////////// */
