/* /////////////////////////////////////////////////////////////////////////////
 * File:        searchspec_sequence_test.cpp
 *
 * Purpose:     Implementation file for the searchspec_sequence_test project.
 *
 * Created:     30th April 2004
 * Updated:     30th May 2004
 *
 * Status:      Wizard-generated
 *
 * License:     (Licensed under the Synesis Software Open License)
 *
 *              Copyright (C) 1999-2004, Synesis Software Pty Ltd.
 *              All rights reserved.
 *
 *              www:        http://www.synesis.com.au/software
 *
 *              email:      software@synesis.com.au
 *
 *              This source code is placed into the public domain 2004
 *              by Synesis Software Pty Ltd. There are no restrictions
 *              whatsoever to your use of the software.
 *
 *              This source code is provided by Synesis Software Pty Ltd "as is"
 *              and any warranties, whether expressed or implied, including, but
 *              not limited to, the implied warranties of merchantability and
 *              fitness for a particular purpose are disclaimed. In no event
 *              shall the Synesis Software Pty Ltd be liable for any direct,
 *              indirect, incidental, special, exemplary, or consequential
 *              damages (including, but not limited to, procurement of
 *              substitute goods or services; loss of use, data, or profits; or
 *              business interruption) however caused and on any theory of
 *              liability, whether in contract, strict liability, or tort
 *              (including negligence or otherwise) arising in any way out of
 *              the use of this software, even if advised of the possibility of
 *              such damage.
 *
 *              Neither the name of Synesis Software Pty Ltd nor the names of
 *              any subdivisions, employees or agents of Synesis Software Pty
 *              Ltd, nor the names of any other contributors to this software
 *              may be used to endorse or promote products derived from this
 *              software without specific prior written permission.
 *
 * ////////////////////////////////////////////////////////////////////////// */

#include <stlsoft.h> // Put here to sort out VC++'s stupid 4786 warning

#include <stdio.h>
#include <stdlib.h>
#if defined(__DMC__) || \
    defined(__WATCOMC__)
# include <iostream.h>
#else /* ? compiler */
# include <iostream>

using std::cout;
using std::cerr;
using std::endl;
#endif /* compiler */

#include <stlsoft.h>
#include <stlsoft_string_access.h>
#include <stlsoft_searchspec_sequence.h>

#if defined(unix) || \
    defined(__unix__)

# include <unixstl.h>
# include <unixstl_findfile_sequence.h>

namespace platform_stl	=	unixstl;

#elif defined(WIN32) || \
    defined(_WIN32)

# include <winstl.h>
# include <winstl_findfile_sequence.h>

namespace platform_stl	=	winstl;

#else /* ? operating system */

# error Operating system not discriminated

#endif /* operating system */

/* ////////////////////////////////////////////////////////////////////////// */

#include <algorithm>

/* ////////////////////////////////////////////////////////////////////////// */

using platform_stl::findfile_sequence;
typedef platform_stl::filesystem_traits<char>	filesystem_traits;

/* ////////////////////////////////////////////////////////////////////////// */

void usage(int bExit);

/* ////////////////////////////////////////////////////////////////////////// */

void print_entry(findfile_sequence::value_type const &entry)
{
	puts(stlsoft::c_str_ptr(entry));
}

/* ////////////////////////////////////////////////////////////////////////// */

#ifndef WIN32
extern "C" void __stdcall Sleep(unsigned long);
#endif // WIN32

int main(int argc, char *argv[])
{
#if 0
	::Sleep(100000);
#endif /* 0 */

    int         iRet		=   0;
    int         i;
	char const	*searchSpec	=	NULL;
	char const	*rootDir	=	NULL;

	puts("searchspec_sequence_test: " __STLSOFT_COMPILER_LABEL_STRING);

	for(i = 1; i < argc; ++i)
	{
		const char	*arg	=	argv[i];

		if(arg[0] == '-')
		{
			if(arg[1] == '-')
			{
				/* -- arguments */
			}
			else
			{
				/* - arguments */
			}
		}
		else
		{
			/* other arguments */
			if(NULL == searchSpec)
			{
				searchSpec = arg;
			}
			else if(NULL == rootDir)
			{
				rootDir = arg;
			}
		}
	}

	/* . */
	if(NULL == searchSpec)
	{
		searchSpec = filesystem_traits::pattern_all();
	}

	if(NULL == rootDir)
	{
		rootDir = "."; // filesystem_traits::path_name_separator()
	}

#if 0
	findfile_sequence	entries(rootDir, searchSpec, findfile_sequence::files);
#else /* ? 0 */
	typedef	stlsoft::searchspec_sequence<findfile_sequence>	sequence_t;

	sequence_t	entries(rootDir, searchSpec, filesystem_traits::path_separator(), findfile_sequence::files);
#endif /* 0 */

	stlsoft_ns_qual_std(for_each)(entries.begin(), entries.end(), print_entry);

	sequence_t::const_iterator	begin	=	entries.begin();
	sequence_t::const_iterator	end		=	entries.end();

	puts("");
	for(; begin != end; ++begin)
	{
		print_entry(*begin);

		sequence_t::const_iterator	next(begin);

		if(++next != end)
		{
			print_entry(*next);
		}
	}

    return iRet;
}

/* ////////////////////////////////////////////////////////////////////////// */

void usage(int bExit)
{
	cerr << endl;
    cerr << "  searchspec_sequence_test" << endl << endl;

#if 0
    cerr << "  searchspec_sequence_test" << endl << endl;
    cerr << "  USAGE: searchspec_sequence_test [{-w | -p<root-paths> | -h}] [-u] [-d] [{-v | -s}] <search-spec>" << endl;
    cerr << "    where:" << endl << endl;
    cerr << "    -w             - searches from the current working directory. The default" << endl;
    cerr << "    -p<root-paths> - searches from the given root path(s), separated by \';\'," << endl;
    cerr << "                     eg." << endl;
    cerr << "                       -p\"c:\\windows;x:\\bin\"" << endl;
    cerr << "    -r             - deletes readonly files" << endl;
    cerr << "    -h             - searches from the roots of all drives on the system" << endl;
    cerr << "    -d             - displays the path(s) searched" << endl;
    cerr << "    -u             - do not act recursively" << endl;
    cerr << "    -v             - verbose output. Prints time, attributes, size and path. (default)" << endl;
    cerr << "    -s             - succinct output. Prints path only" << endl;
    cerr << "    <search-spec>  - one or more file search specifications, separated by \';\'," << endl;
    cerr << "                     eg." << endl;
    cerr << "                       \"*.exe\"" << endl;
    cerr << "                       \"myfile.ext\"" << endl;
    cerr << "                       \"*.exe;*.dll\"" << endl;
    cerr << "                       \"*.xl?;report.*\"" << endl;
	cerr << endl;
	cerr << "  Contact " << _nameSynesisSoftware << endl;
	cerr << "    at \"" << _wwwSynesisSoftware_SystemTools << "\"," << endl;
	cerr << "    or, via email, at \"" << _emailSynesisSoftware_SystemTools << "\"" << endl;
	cerr << endl;
#endif /* 0 */

    if(bExit)
    {
        exit(1);
    }
}

/* ////////////////////////////////////////////////////////////////////////// */
