// findfile_sequence_test.cpp
//
// Updated: 11th September 2005

// This will cause various compile-time messages to be emitted. When you get
// sick of them just comment out or remove this #define
#define	_STLSOFT_COMPILE_VERBOSE

#include <stlsoft.h>

#include <winstl.h>

#if defined(__STLSOFT_COMPILER_IS_MSVC) && \
    _MSC_VER == 1100
# pragma warning(disable : 4550) // VC 5 is a bit dim
#endif /* _MSC_VER == 1100 */

//#define _WINSTL_FINDFILE_ITERATOR_IS_FORWARD

#include <stlsoft_simple_algorithms.h>

#include <winstl_findfile_sequence.h>

winstl_ns_using(findfile_sequence_a)
winstl_ns_using(findfile_sequence_w)

#include <winstl_current_directory.h>

winstl_ns_using(current_directory_a)
winstl_ns_using(current_directory_w)

#include <winstl_windows_directory.h>

winstl_ns_using(windows_directory_a)
winstl_ns_using(windows_directory_w)

#include <stdio.h>
#include <wchar.h>

#include <functional>
#include <algorithm>

static void dump_path(char const *path);

// Because of the difficulties that Borland and GNU compilers have with
// these definitions, this functional looks extremely complex. When
// writing for a single compiler, or for a set that includes the 'better'
// compilers (Intel, Metrowerks, Comeau, Digital Mars) it would not be
// an issue.

struct print_path
{
public:
	void operator ()(winstl_ns_qual(findfile_sequence_a)::value_type const &value)
	{
		fprintf(stdout, "  %s (%s, %s)\n", (const char *)stlsoft_ns_qual(c_str_ptr)(value), value.get_filename(), value.get_short_filename());
	}
	void operator ()(winstl_ns_qual(findfile_sequence_w)::value_type const &value)
	{
		fwprintf(stdout, L"  %s (%s, %s)\n", (const wchar_t *)value, value.get_filename(), value.get_short_filename());
	}
};


void dump_path(char const *path)
{
	printf("  %s\n", path);
}

int main(int argc, char **argv)
{
#if 0
	for(size_t i = 0; i < 0xffffffff; ++i){}
#endif /* 0 */

	{
#if 1
		current_directory_a	current_dir;
# if 0
		findfile_sequence_a	files(current_dir, "*.*", findfile_sequence_a::files | findfile_sequence_a::includeDots);
# else /* ? 0 */
		char const			*dir	=	"NULL";
		findfile_sequence_a	files(dir, "*.*", findfile_sequence_a::files | findfile_sequence_a::includeDots);
# endif /* 0 */
#else /* ? 0 */
# if 1
		using winstl::file_path_buffer_a;

		current_directory_a	current_dir;
		file_path_buffer_a	upOneDir;
		file_path_buffer_a	pattern;
		char				*filePart;
		findfile_sequence_a::traits_type::get_full_path_name(current_dir.c_str(), upOneDir.size(), &upOneDir[0], &filePart);
		*(filePart - 1) = '\0';
		findfile_sequence_a::traits_type::str_copy(&pattern[0], filePart);
		findfile_sequence_a::traits_type::ensure_dir_end(&pattern[0]);
		findfile_sequence_a::traits_type::str_cat(&pattern[0], "*.*");

		findfile_sequence_a	files(upOneDir.c_str(), pattern.c_str(), findfile_sequence_a::files | findfile_sequence_a::includeDots);
# else /* ? 0 */
		findfile_sequence_a	files("*.*", findfile_sequence_a::files | findfile_sequence_a::includeDots);
# endif /* 0 */
#endif /* 0 */

		printf("\nCurrent files (ANSI, via implicit conversion to char const *): dir=%s\n", files.get_directory());
		stlsoft_ns_qual(for_each_preinc)(files.begin(), files.end(), dump_path);

		printf("\nCurrent files (ANSI, via sequence::value_type): dir=%s\n", files.get_directory());
		stlsoft_ns_qual(for_each_preinc)(files.begin(), files.end(), print_path());

		printf("\nCurrent files (ANSI, via implicit conversion to char const *, post-increment operator): dir=%s\n", files.get_directory());
		stlsoft_ns_qual(for_each_postinc)(files.begin(), files.end(), dump_path);

		printf("\nCurrent files (ANSI, via sequence::value_type, post-increment operator): dir=%s\n", files.get_directory());
		stlsoft_ns_qual(for_each_postinc)(files.begin(), files.end(), print_path());
	}

	{
		current_directory_a	current_dir;
//		findfile_sequence_a	files(current_dir, "F:\\DOCUME~1\\Matty\\LOCALS~1\\Temp\\./ffs-test/dir0.ffs/./../dir0.ffs/file0?.ffs", findfile_sequence_w::files);
		findfile_sequence_a	files(current_dir, "F:\\DOCUME~1\\Matty\\LOCALS~1\\Temp\\ffs-test\\./dir0.ffs/./../dir0.ffs/file0?.ffs", findfile_sequence_w::files);


		printf("\nUnit-test directory files (ANSI, via implicit conversion to char const *): dir=%s\n", files.get_directory());
		stlsoft_ns_qual(for_each_preinc)(files.begin(), files.end(), dump_path);
	}

	{
		current_directory_w	current_dir;
		findfile_sequence_w	files(current_dir, L"*.*", findfile_sequence_w::files);

		wprintf(L"\nCurrent files (Unicode, via sequence::value_type): dir=%s\n", files.get_directory());
		stlsoft_ns_qual(for_each_preinc)(files.begin(), files.end(), print_path());
	}

	{
		windows_directory_a	win_dir;
		findfile_sequence_a	directories(win_dir, "*.*", findfile_sequence_a::directories);

		printf("\nWindows directory directories (ANSI, via implicit conversion to char const *): dir=%s\n", directories.get_directory());
		stlsoft_ns_qual(for_each_preinc)(directories.begin(), directories.end(), dump_path);
	}

#if _STLSOFT_VER >= 0x010804
	{
		windows_directory_a	win_dir;
		findfile_sequence_a	directories(win_dir, "system32\\*.exe;system\\*.dll", ';', findfile_sequence_a::files);

		printf("\nWindows directory files (ANSI, via implicit conversion to char const *): dir=%s\n", directories.get_directory());
		stlsoft_ns_qual(for_each_preinc)(directories.begin(), directories.end(), dump_path);
	}
#endif /* 0 */

#if _STLSOFT_VER >= 0x010805
	{
		windows_directory_a	win_dir;
		findfile_sequence_a	directories(win_dir, ";;*.dll;;;*.exe;;;;;;;", ';', findfile_sequence_a::files);

		printf("\nWindows directory directories (ANSI, via implicit conversion to char const *): dir=%s\n", directories.get_directory());
		stlsoft_ns_qual(for_each_preinc)(directories.begin(), directories.end(), print_path());
	}
#endif /* 0 */

#if _STLSOFT_VER >= 0x010805
	{
		current_directory_a	current_dir;
		findfile_sequence_a	directories(current_dir, ";.;..", ';', findfile_sequence_a::directories);

		printf("\nCurrent directory current and parent directories (ANSI, via implicit conversion to char const *): dir=%s\n", directories.get_directory());
		stlsoft_ns_qual(for_each_preinc)(directories.begin(), directories.end(), print_path());
	}
#endif /* 0 */

	{
		windows_directory_w	win_dir;
		findfile_sequence_w	files(win_dir, L"*.*", findfile_sequence_w::files);

		wprintf(L"\nWindows directory files (Unicode, via sequence::value_type): dir=%s\n", files.get_directory());
		stlsoft_ns_qual(for_each_preinc)(files.begin(), files.end(), print_path());
	}

	// Test operator ++(int) - for Tanemura

	{
		windows_directory_w	win_dir;
		findfile_sequence_w	files(win_dir, L"*.exe", findfile_sequence_w::files);

		wprintf(L"\nWindows directory files (Unicode, via sequence::value_type, post-increment operator): dir=%s\n", files.get_directory());
		findfile_sequence_w::const_iterator	begin	=	files.begin();
		findfile_sequence_w::const_iterator	end		=	files.end();

		for(; begin != end; )
		{
			print_path()(*begin++);
		}
	}

	{
		windows_directory_a	win_dir;
		findfile_sequence_a	files(win_dir, "*.exe", findfile_sequence_a::files);

		printf("\nWindows directory files (ANSI, copying iterators): dir=%s\n", files.get_directory());
		findfile_sequence_a::const_iterator	begin	=	files.begin();
		findfile_sequence_a::const_iterator	end		=	files.end();

		for(; begin != end; ++begin)
		{
			print_path()(*begin);

			findfile_sequence_a::const_iterator	next(begin);

			if(++next != end)
			{
				print_path()(*next);
			}
		}
	}

	{
		findfile_sequence_a	files(".", ".", findfile_sequence_a::directories);

		printf("\nSpecial case '.' pattern (ANSI): dir=%s\n", files.get_directory());
		findfile_sequence_a::const_iterator	begin	=	files.begin();
		findfile_sequence_a::const_iterator	end		=	files.end();

		for(; begin != end; ++begin)
		{
			print_path()(*begin);
		}
	}

	return 0;
}

/* ////////////////////////////////////////////////////////////////////////// */
